-------------------------------------------------------------------------------
-- Lorewalkers by Lockslap, US-Bleeding Hollow
-- http://ba-guild.com
-------------------------------------------------------------------------------
local Lorewalkers	= LibStub("AceAddon-3.0"):NewAddon("Lorewalkers", "AceEvent-3.0", "AceConsole-3.0")
local L				= LibStub("AceLocale-3.0"):GetLocale(Lorewalkers.name, false)
if not TomTom then return end

-- setup some globals
Lorewalkers.name 	= GetAddOnMetadata("Lorewalkers", "Title")
Lorewalkers.version	= GetAddOnMetadata("Lorewalkers", "Version")
Lorewalkers.db		= {}

local curZone
local waypoints		= {}
local zones			= {}
local defaults 		= {
	char = {
		enabled = true,
		exalted	= false,
	}
}

local enabledString		= ("|cff00ff00%s|r"):format(L["Enabled"])
local disabledString	= ("|cffff0000%s|r"):format(L["Disabled"])

local function ZoneName(str)
	return str:lower():gsub(" ", "")
end

local function CheckReputation()
	-- check if you're exalted with the lorewalkers (id is 1345)
	local _, _, standing, _, _, _, _, _, _, _, _, _, _ = GetFactionInfoByID(1345)
	Lorewalkers.db.char.exalted = standing == 8 and true or false
	return Lorewalkers.db.exalted
end

local function LoadCriteria()
	for title, id in pairs(Lorewalkers.achievements) do
		for i = 1, GetAchievementNumCriteria(id) do
			-- get the achievement criteria
			local description, _, completed, _, _, _, _, _, _, _ = GetAchievementCriteriaInfo(id, i)
			Lorewalkers.criterias[description] = completed
		end
	end
end

local function ClearWaypoints()
	if #waypoints == 0 then return end
	for _, point in pairs(waypoints) do
		TomTom:RemoveWaypoint(point)
	end	
	waypoints = {}
	TomTom:ReloadWaypoints()
end

local function GetNumWaypoints(zone)
	zone = ZoneName(zone)
	if not zone or not Lorewalkers.waypoints[zone] then return 0 end
	local i = 0
	for _, _ in pairs(Lorewalkers.waypoints[zone]) do
		i = i + 1
	end
	return i
end

local function ToggleAddon()
	--CheckReputation()
	if not Lorewalkers.db.char.exalted then
		Lorewalkers.db.char.enabled = not Lorewalkers.db.char.enabled
		Lorewalkers:Print((L["Waypoints have been %s"]):format(Lorewalkers.db.char.enabled == true and enabledString or disabledString))
		if not Lorewalkers.db.char.enabled then
			ClearWaypoints()
		else
			curZone = ZoneName(GetZoneText())
			Lorewalkers:UpdateWaypoints(curZone)
		end
	elseif not Lorewalkers.db.char.enabled then
		-- player is exalted
		Lorewalkers.db.char.enabled = false
		Lorewalkers:Print((L["Waypoints automatically disabled because you're exalted with |cff1784d1The Lorewalkers|r"]))
		if #waypoints > 0 then ClearWaypoints() end
	end
end

function Lorewalkers:UpdateWaypoints(zone)
	local loaded, completed = 0, 0
	local completedString = "|cff00ff00(%d %s)|r"
	if self.waypoints[zone] and zones[zone] then	
		-- first clear all of our waypoints
		ClearWaypoints()
		for desc, point in pairs(self.waypoints[zone]) do
			-- get the criteria name
			local _, cName = desc:match("(.+): (.+)")
			if not Lorewalkers.criterias[cName] then	-- the criteria wasnt found, or it doesnt exist
				loaded = loaded + 1
				waypoints[#waypoints + 1] = TomTom:AddZWaypoint(6, zones[zone], point.x, point.y, desc, false, false, false, false, false, true)
			else
				completed = completed + 1
			end
		end
		
		-- print completed/loaded information
		if loaded > 0 then
			self:Print((L["Loaded |cff1784d1%d|r %s waypoints for |cff1784d1%s|r"]):format(loaded, completedString:format(completed, L["completed"]), GetZoneText()))
		else
			self:Print((L["You have completed all waypoints for |cff1784d1%s|r"]):format(GetZoneText()))
		end
	end
end

function Lorewalkers:HandleEvents(event, ...)
	curZone = ZoneName(GetZoneText())
	if event == "PLAYER_ENTERING_WORLD" then
		-- load the config
		self.db = LibStub("AceDB-3.0"):New("LorewalkersDB", defaults)
		
		-- load the achievement criteria
		LoadCriteria()
		
		-- check reputation
		--CheckReputation()
		
		for id, name in pairs{ GetMapZones(6) } do
			zones[ZoneName(name)] = id
		end
		
		-- check if they're exalted already
		if self.db.char.exalted == true and self.db.char.enabled == true then
			self.db.char.enabled = false
			self:Print((L["Waypoints automatically disabled because you're exalted with |cff1784d1The Lorewalkers|r"]))
		end

		-- set tomtom arrival distance to 5 yards
		TomTom.profile.arrow.arrival = 5
		
		-- register the chat commands to toggle the addon
		self:RegisterChatCommand("lorewalkers", function() ToggleAddon() end)
		self:RegisterChatCommand("lw", function() ToggleAddon() end)
		
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end
	
	if self.db.char.enabled == true and not self.db.char.exalted then
		self:UpdateWaypoints(curZone)
	end
end

function Lorewalkers:CHAT_MSG_COMBAT_FACTION_CHANGE()
	-- check if exalted with the lorewalkers
	--CheckReputation()
	if self.db.char.exalted == true then
		-- exalted, so disable waypoints
		self.db.char.enabled = false
		self:Print((L["Waypoints automatically disabled because you're exalted with |cff1784d1The Lorewalkers|r"]))
		if #waypoints > 0 then ClearWaypoints() end
	end
end

Lorewalkers:RegisterEvent("ZONE_CHANGED_NEW_AREA", "HandleEvents")
Lorewalkers:RegisterEvent("PLAYER_ENTERING_WORLD", "HandleEvents")
Lorewalkers:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")	-- for exalted check

-- achievements
Lorewalkers.achievements = {
	[L["Ballad of Liu Lang"]]					= 6856,
	[L["Between a Saurok and a Hard Place"]]	= 6716,
	[L["Fish Tails"]]							= 6846,
	[L["Heart of the Mantid Swarm"]]			= 6857,
	[L["Hozen in the Mist"]]					= 6850,
	[L["Legend of the Brewfathers"]]			= 7230,
	[L["The Dark Heart of the Mogu"]]			= 6754,
	[L["The Seven Burdens of Shaohao"]]			= 6855,
	[L["The Song of the Yaungol"]]				= 6847,
	[L["What is Worth Fighting For"]]			= 6858,
}

-- criteria
Lorewalkers.criterias = {}

-- waypoints for tomtom
Lorewalkers.waypoints		= {
	[ZoneName(L["Dread Wastes"])] = {
		[L["Heart of the Mantid Swarm: Cycle of the Mantid"]]	= {
			x	= 48.38,
			y	= 32.85,
		},
		[L["Heart of the Mantid Swarm: The Empress"]] = {
			x	= 35.53,
			y	= 32.61,
		},
		[L["Heart of the Mantid Swarm: Mantid Society"]] = {
			x	= 59.90,
			y	= 54.70,
		},
		[L["Heart of the Mantid Swarm: Amber (Entrance)"]] = {
			x	= 53.61,
			y	= 15.48,
		},
		[L["Heart of the Mantid Swarm: Amber"]] = {
			x	= 52.52,
			y	= 10.06,
		},
		[L["Between a Saurok and a Hard Place: The Deserter"]] = {
			x	= 67.50,
			y	= 60.90,
		},
	},
	
	[ZoneName(L["Krasarang Wilds"])] = {
		[L["Ballad of Liu Lang: Waiting for the Turtle"]] = {
			x	= 72.21,
			y	= 31.01,
		},
		[L["Fish Tails: Origins"]] = {
			x	= 30.55,
			y	= 38.57,
		},
		[L["The Seven Burdens of Shaohao: The Emperor's Burden - Part 4"]] = {
			x	= 40.50,
			y	= 56.62,
		},
		[L["Legend of the Brewfathers: Quan Tou the Two Fisted"]] = {
			x	= 81.43,
			y	= 11.45,
		},
		[L["Hozen in the Mist: Hozen Maturiy"]] = {
			x	= 52.39,
			y	= 87.66,
		},
		[L["Between a Saurok and a Hard Place: The Last Stand"]] = {
			x	= 32.78,
			y	= 29.41,
		},
		[L["The Dark Heart of the Mogu: The Lost Dynasty"]] = {
			x	= 50.94,
			y	= 31.69,
		},
	},
	
	[ZoneName(L["Kun-Lai Summit"])] = {
		[L["Fish Tails: Role Call"]] = {
			x	= 74.48,
			y	= 83.55,
		},
		[L["The Seven Burdens of Shaohao: The Emperor's Burden - Part 2"]] = {
			x	= 43.82,
			y	= 51.19,
		},
		[L["The Seven Burdens of Shaohao: The Emperor's Burden - Part 6"]] = {
			x	= 67.76,
			y	= 48.33,
		},
		[L["The Seven Burdens of Shaohao: The Emperor's Burden - Part 7"]] = {
			x	= 40.90,
			y	= 42.50,
		},
		[L["What is Worth Fighting For: Victory in Kun-Lai"]] = {
			x	= 63.04,
			y	= 40.82,
		},
		[L["Legend of the Brewfathers: Ren Yun the Blind"]] = {
			x	= 44.70,
			y	= 52.37,
		},
		[L["Hozen in the Mist: The Hozen Ravage"]] = {
			x	= 45.76,
			y	= 61.90,
		},
		[L["The Song of Yaungol: Yaungoil"]] = {
			x	= 71.72,
			y	= 63.02,
		},
		[L["The Dark Heart of the Mogu: Valley of the Emperors (Entrance)"]] = {
			x	= 52.90,
			y	= 46.60,
		},
		[L["The Dark Heart of the Mogu: Valley of the Emperors"]] = {
			x	= 58.70,
			y	= 72.60,
		},
	},
	
	[ZoneName(L["The Jade Forest"])] = {
		[L["Fish Tails: Watersmithing"]] = {
			x	= 66.01,
			y	= 87.56
		},
		[L["The Seven Burdens of Shaohao: The Emperor's Burden - Part 1"]] = {
			x	= 47.08,
			y	= 45.14,
		},
		[L["The Seven Burdens of Shaohao: The Emperor's Burden - Part 3"]] = {
			x	= 55.88,
			y	= 56.85,
		},
		[L["What is Worth Fighting For: The First Monks"]] = {
			x	= 35.74,
			y	= 30.46,
		},
		[L["Legend of the Brewfathers: Xin Wo Yin the Broken Hearted"]] = {
			x	= 37.30,
			y	= 30.12,
		},
		[L["Hozen in the Mist: Hozen Speech"]] = {
			x	= 26.38,
			y	= 28.33,
		},
		[L["Between a Saurok and a Hard Place: The Saurok"]] = {
			x	= 67.72,
			y	= 29.35,
		},
		[L["The Dark Heart of the Mogu: Spirit Binders"]] = {
			x	= 42.26,
			y	= 17.47,
		},
	},
	
	[ZoneName(L["The Veiled Stair"])] = {
		[L["Between a Saurok and a Hard Place: The Defiant"]] = {
			x	= 55.00,
			y	= 16.11,
		},
	},
	
	[ZoneName(L["Townlong Steppes"])] = {
		[L["The Seven Burdens of Shaohao: The Emperor's Burden - Part 5"]] = {
			x	= 37.74,
			y	= 62.91,
		},
		[L["The Song of Yaungol: Trapped in a Strange Land"]] = {
			x	= 84.08,
			y	= 72.86,
		},
		[L["The Song of Yaungol: Dominance"]] = {
			x	= 65.50,
			y	= 50.10,
		},
	},
	
	[ZoneName(L["Vale of Eternal Blossoms"])] = {
		[L["The Seven Burdens of Shaohao: The Emperor's Burden - Part 8"]] = {
			x	= 68.80,
			y	= 44.22,
		},
		[L["What is Worth Fighting For: Always Remember"]] = {
			x	= 52.93,
			y	= 68.65,
		},
		[L["What is Worth Fighting For: Together, We Are Strong"]] = {
			x	= 26.62,
			y	= 21.49,
		},
		[L["The Dark Heart of the Mogu: The Thunder King"]] = {
			x	= 40.24,
			y	= 77.48
		},
	},
	
	[ZoneName(L["Valley of the Four Winds"])] = {
		[L["Ballad of Liu Lang: The Birthplace of Liu Lang"]] = {
			x	= 20.25,
			y	= 55.86,
		},
		[L["Ballad of Liu Lang: The Wandering Widow"]] = {
			x	= 34.57,
			y	= 63.87,
		},
		[L["Ballad of Liu Lang: A Most Famous Bill of Sale"]] = {
			x	= 55.09,
			y	= 47.13,
		},
		[L["Fish Tails: Waterspeakers"]] = {
			x	= 61.22,
			y	= 34.69,
		},
		[L["What is Worth Fighting For: Pandaren Fighting Tactics"]] = {
			x	= 18.84,
			y	= 31.70,
		},
		[L["Hozen in the Mist: Embracing the Passions"]] = {
			x	= 83.19,
			y	= 21.18,
		},
	},
}
